;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xa1579afb2cb0611327f7a3c507588bda . "./code.cg2js/cg-status-bar.fasl")
(#x40513688956e379077a40070c828a827 . "./code.cg2js/cg-hotspot.fasl")
(#x4b19ccbc4424ee0f9f5e69c5962d0059 . "./code.cg2js/cg-toolbar.fasl")
(#x05ddf1d78017bfdec40f40970ffb976e . "./code.cg2js/cg-clipboard-stack.fasl")
(#x3b7ad232c5a7ab091300f5befd4c95f2 . "./code.cg2js/cg-tooltip.fasl")
(#x33dad4c03dda4a5eab04babbb2bb8bec . "./code.cg2js/cg-calendar.fasl")
(#x156a986fa52c2174d4f58b0c80e7b5a8 . "./code.cg2js/cg-item-list.fasl")
(#x2772e5ec72bff3e21a6a3fcf45f7b215 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x80de3571c7c1b9a533cca02218e94bc4 . "./code.cg2js/cg-trackbar.fasl")
(#x2c5311fa3f686584972aaebf3ff4b8db . "./code.cg2js/cg-class-grid.fasl")
(#xa7eaba75300beafd7e993736ade92e87 . "./code.cg2js/cg-dialog-item.fasl")
(#x98cd630936b220fe23949b00f505af2b . "./code.cg2js/cg-comtab.fasl")
(#xb271dd11c7d49087f748c270c40a70f0 . "./code.cg2js/cg-lisp-text.fasl")
(#xd37da50d0ac4198f337d6119d713e2d1 . "./code.cg2js/cg-button.fasl")
(#xc33b4475b755b07e2f26bf4723887e5a . "./code.cg2js/cg-printing.fasl")
(#xb032c51b3f4b77b6e4b14e7775039902 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x5c6bb071e71048c3d1cb3bb683654591 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x586de0ff20f30780bfb459f0a8c5a765 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xe29a7eeb5cf492dd3fd707813716e210 . "./code.cg2js/cg-selected-object.fasl")
(#x87b902e57679b9867d039f0e0fe57ce1 . "./code.cg2js/cg-text-or-combo.fasl")
(#x5def7c259ae4bcf58c2a3c1762d8bd56 . "./code.cg2js/cg-graphics-context.fasl")
(#x96923e086b56605c2df7d10227902b0e . "./code.cg2js/cg-edit-in-place.fasl")
(#x2c252ef9db81b7556b10ba51059fc8ae . "./code.cg2js/cg-string-dialog.fasl")
(#x598a5d5d126cd9f0954868c963b375cb . "./code.cg2js/cg-list-view.fasl")
(#x060d7a342ca9f3ecd3a0bf7c4c35d8e0 . "./code.cg2js/cg-intersect.fasl")
(#xf2c57dd6f4aa00d184d8f6444ec8c6e3 . "./code.cg2js/cg-text-widget.fasl")
(#x8f30adcddfdf7f1573f8e88c7167922c . "./code.cg2js/cg-directory-dialog.fasl")
(#x89bc0c920ce92b37d6b8e1d7b2e257a2 . "./code.cg2js/cg-file-dialog.fasl")
(#xfd64fe5788ea24ac5f9d37d5e5b2a998 . "./code.cg2js/cg-dde-utils.fasl")
(#xad1607b9321f622e766be7f67ffb0cb6 . "./code.cg2js/cg-nodes.fasl")
(#x9fa118164a79fe7880d3626be2a09f02 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x34882ef6cf37efe5514ede76561c2146 . "./code.cg2js/cg-up-down-control.fasl")
(#xb2f2bc0c3de200dcfe88c98328d62634 . "./code.cg2js/cg-get-position.fasl")
(#xd07948428aa86ef99cb3fbfe2c1416a0 . "./code.cg2js/cg-toggling-widget.fasl")
(#xee48e74dc8ac4d2853841ab7460ea341 . "./code.cg2js/cg-acache.fasl")
(#x43cbc46076e3df64b40068b0e71b9a90 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x2e397975755b7f20d905d743d6536973 . "./code.cg2js/cg-base.fasl")
(#xbae119bee1adbaba7305e6eddc9f4833 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x4f12271b2236834e0d7845a1ef625e2d . "./code.cg2js/cg-find-string-dialog.fasl")
(#x118cf3d6bef986b8bdc7f993da2dee83 . "./code.cg2js/cg-property.fasl")
(#x137bee0bfda7bdb56f77e438a2eb0f8c . "./code.cg2js/cg-editable-text.fasl")
(#xdcffcb0aae2787d8782242fc1236296c . "./code.cg2js/cg-lamp.fasl")
(#x4f87da0c31bb423b9ae80e4a2a4eac69 . "./code.cg2js/cg-progress-indicator.fasl")
(#xb14131f62b8c433800ceaaa6cee0f49f . "./code.cg2js/cg-chart-widget.fasl")
(#x0c9c2cdf2a2dd1eaced6d8dab87fe1b3 . "./code.cg2js/cg-common-control.fasl")
(#x26c074065360f50f618a4aac540db221 . "./code.cg2js/cg-font-dialog.fasl")
(#x32ca451e753939243aad869de8575903 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x07369d771c51e837484147b9e3ee566c . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x280ea9ee6d30e28a619d3c35a30c3590 . "./code.cg2js/cg-os-window.fasl")
(#x0aa38a2661b6724399e2171c3f1d4f6d . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x6dfe9d1e3b07c61a5cd883e5b42db8e7 . "./code.cg2js/cg-pixmap.fasl")
(#x3b7711a4bcf8bc851051de0c82d404b1 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x6dd3fb7bb7afd2f590f5c7a197e5f20a . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xa2911d8efbd9d89abc97628648fd60ac . "./code.cg2js/cg-scaling-stream.fasl")
(#x65787ebf899020dfff730bc2f1e8c3d9 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x588f962edd5378c74d20de59c7ce1a13 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x105c403efecec878da4ccc4fc4523f9c . "./code.cg2js/cg-timer.fasl")
(#x554044cc465f16e7004c08475d74872e . "./code.cg2js/cg-scroll-bar.fasl")
(#x00ca2185fddb021a803351afb9a250ff . "./code.cg2js/cg-caret.fasl")
(#xe453b8be7618594f2b4d21b995179756 . "./code.cg2js/cg-template-string.fasl")
(#x1f9e34adaf4a0752d9d55d3a362832a7 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xa35640c1bc1c7248fa4fea80d927bc64 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xec8bd6aad2da1b73c045ecc2971acf2c . "./code.cg2js/cg-combo-box.fasl")
(#x23732b694fa1ef6afb02bd731883bbd3 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x6c6f2e8f8240f73e265c9b9845eb5d39 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x39917bbcf848bc13056743b4dd319a6b . "./code.cg2js/cg-menu-tooltip.fasl")
(#x1775451e263c3e2144668cfe05e97367 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x04b17458539116f496aea7aed66a2fef . "./code.cg2js/cg-player.fasl")
(#x96b1c6d00ca79bfa2fb8283fb17fc104 . "./code.cg2js/cg-outline.fasl")
(#x55bdbad00ddac48d3762425005e4a86c . "./code.cg2js/cg-class-support.fasl")
(#x7ea7f3aeecef779bf3196c6945ea7088 . "./code.cg2js/cg-lettered-menu.fasl")
(#x4ea56d45ea437151ed72a0e846112e4c . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x90740516ada4779f253d03a05d16f99d . "./code.cg2js/cg-fill-texture.fasl")
(#x1245eccab14cbdb74ee310b00a211af6 . "./code.cg2js/cg-lisp-widget.fasl")
(#xf46f106feae9c1718dc262852c8d10d3 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xe105d9e56c0ccf3d5a632f1e850ba611 . "./code.cg2js/cg-clipboard.fasl")
(#x3e26d07168c480e6ddefc3fcdb96aa41 . "./code.cg2js/cg-wrap-string.fasl")
(#xaddafc408e1daf4352203b19ad5c0193 . "./code.cg2js/cg-os-widget.fasl")
(#x28e19ba5d70220ff183e7c5d6cb2c0a3 . "./code.cg2js/cg-check-box.fasl")
(#xa4d0b14f00c94419f6596ae0a9774e5a . "./code.cg2js/cg-choice-list.fasl")
(#x021093bdf1cedf0ed6da27d1ca00d483 . "./code.cg2js/cg-layout.fasl")
(#x9fd024c056bba9f1cf1098a8ef3cd854 . "./code.cg2js/cg-group-box.fasl")
(#xe06b66086c09160c46bed49e4cb8a5d4 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x06c7fd3c24c653c08e34bc2be76a4ebe . "./code.cg2js/cg-radio-button.fasl")
(#x0fff38c52fbce619ed63aff55eae825d . "./code.cg2js/cg-tab-control.fasl")
(#x92927467728b15de37b264434d3cc832 . "./code.cg2js/cg.fasl")
(#x68bc57f117808795076662b6e3b2123d . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xe82d882c9a3dec6c26a1c088745bc47a . "./code.cg2js/cg-dropping-outline.fasl")
(#xb4b2e3234758478b280292cd9b258052 . "./code.cg2js/cg-get-pixmap.fasl")
(#xf2a2eb9dd46e748ff934b63db40f7939 . "./code.cg2js/cg-grid-widget.fasl")
(#xe8b08206705002ba98d28705d4322e18 . "./code.cg2js/cg-utility-dialog.fasl")
(#x0ec7e7c5b46c492de4c98a1749961e5e . "./code.cg2js/cg-static-text.fasl")
(#x9998af9c7c522ca384f12e9b23628c0b . "./code.cg2js/cg-class-slot-grid.fasl")
(#xd9fddf0d9ecac61b2d6cb9f6bd8502b6 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x74f1afa6390e72012e4706ce775e6a50 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xdeea0f6c362593ca131692bf53fbe02e . "./code.cg2js/cg-bitmap-pane.fasl")
(#xdef0404c4bc12dbbe137056f2e2d3234 . "./code.cg2js/cg-plot-widget.fasl")
(#xe6fd7df10b71ff9b9443fa9d7226d59f . "./code.cg2js/cg-color-dialog.fasl")
(#x598a6302ccc31dba889506b4134ae2c3 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x1913f4502a14cd5736b8ab5e98c4f491 . "./code.cg2js/cg-icon.fasl")
(#xb10eb849989764198ce5cbe9f9a3bd4d . "./code.cg2js/cg-dev.fasl")
(#x89169521188d5e56dce09b69c6b108d0 . "./code.cg2js/cg-choose-printer.fasl")
(#xb8fa3e15593826c27a39f5f0089ded48 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xabc08ab6514e0135e9dca6ac01b04a49 . "./code.cg2js/cg-menu.fasl")
(#xa46c4a4d3041c189629a1901e01e1875 . "./code.cg2js/cg-yes-no-list.fasl")
(#xe5aef1c54d734ca39018c140a1a028f5 . "./code.cg2js/cg-palette.fasl")
(#x1ea03913cb88349f19eef19c56d7cd9b . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xaba3bec82f472b1efb32be7a7c04c6df . "./code.cg2js/cg-message-dialog.fasl")
(#x3a07bf15caaae71b06baa73a1fbeeacb . "./code.cg2js/cg-shortcut-menu.fasl")
(#x4559236263ea12b49f381f79cd1cca74 . "./code.cg2js/cg-split-bar.fasl")
(#x645aed226279f94c8510c94d95f2cad1 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x995a043c45b93ac36825b153b895a568 . "./code.cg2js/cg-web-browser.fasl")
(#x3a751f77a009299f25897a140140410a . "./code.cg2js/cg-project-window.fasl")
(#x46b138073af8f8b7d2cc7e0094135ddb . "./code.cg2js/cg-picture-widget.fasl")
(#x7e92bd54bd4448d22c4cccb79640d3b5 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x5cd2fdbf0bc9bf33d207e52529923a6f . "./code.cg2js/cg-object-editor.fasl")
(#x3bdf26e5ef5276dd50bcf13c4b316108 . "./code.cg2js/cg-paren-matching.fasl")
(#x58c4098d16f86f29548213cec56fec5a . "./code.cg2js/cg-sample-file-menu.fasl")
(#x61af86529d2cbda4c0b77b82b66719a3 . "./code.cg2js/cg-drawable.fasl")
(#xb1cba27d378fef17388938be773533d6 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x43d35cadc853ff3bfac97e4de34415f1 . "./code.cg2js/cg-html-widget.fasl")
(#x45521cd2289c6d57ed839fad26eb92dc . "./code.cg2js/cg-curve.fasl")
(#x2d1e111fef5aacec5e116ce033bcf09f . "./code.cg2js/cg-yes-no-string.fasl")
(#x30edb4f360cf236b0c729244868f913c . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xd36cbd134b27e11303867ef65e6be280 . "./code.cg2js/cg-html-dialog.fasl")
(#x69696dd9cac0c117f6bfc3901bf7fe1d . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x18de8a95b806b1af76149a62d7b95ea3 . "./code.cg2js/cg-tray.fasl")
(#xa50ad561b13b45d43ad9be1e64ccff2e . "./code.idejs/ide-apropos.fasl")
(#x34a974e4e1a82d336410e7965fa5a48b . "./code.idejs/ide-window-list.fasl")
(#xce71c1615c3a33e63abbc34bfa04dec1 . "./code.idejs/ide-class-browser.fasl")
(#xd35798f7d0bbf2ad56a38a3fabf55ac3 . "./code.idejs/ide-base.fasl")
(#x7c86fa7a793ad54153f9736b88fd2dac . "./code.idejs/ide-process-browser.fasl")
(#xf587a768382a6b4b21f773ce8089b06f . "./code.idejs/ide-acache.fasl")
(#x39236e2d7eb20bf3dae9f110ec416ff7 . "./code.idejs/ide-navigator.fasl")
(#x3ba9c3f81ca54907084c966cd1424d40 . "./code.idejs/ide-trace.fasl")
(#xad37baff3bd028497888e65464aff38b . "./code.idejs/ide-project.fasl")
(#x268ebc5e3d3fc772054fea4bb69c7380 . "./code.idejs/ide-patch-dialog.fasl")
(#x48a065c558e6c008b47d4d59587b8fa6 . "./code.idejs/ide-grapher.fasl")
(#x65f7aa4d1fe64bd3018c1e8c1852ae01 . "./code.idejs/ide-dev.fasl")
(#xbf08e2ced748ba0d0bb0fcc4b546df63 . "./code.idejs/ide-keyshort.fasl")
(#xf16f2464508b9e8ce8724a0158d406d1 . "./code.idejs/ide-editor.fasl")
(#xb72e989290f94876b83469c54051fabe . "./code.idejs/ide-inspector.fasl")
(#xd7d9ec7f946c6bae6efc55660da9ad8a . "./code.idejs/ide.fasl")
(#xec1cb9cecaf3640f1bed58ebbe39ee40 . "./code.idejs/ide-clipboard.fasl")
(#x2f397346ac69932f36833429171ff122 . "./code.idejs/ide-find-definitions.fasl")
(#x9082510b3cea01361d476ef3aced4668 . "./code.idejs/ide-form.fasl")
(#x651244d7c5fd1d1844b23d2a9accd3fe . "./code.idejs/ide-intro.fasl")
(#x3120aaefd831f57fc66cc55412f795be . "./code.idejs/ide-find-in-files.fasl")
(#xb862354ed27f1c8a1d1fdc2ea4d132d4 . "./code.idejs/ide-package-list.fasl")
(#x0be85ad779c58a0a5c9a6360c0b95606 . "./code.idejs/ide-class-grapher.fasl")
(#x4a9bad5a7becdca3247e09ac504560a1 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
